/*This file is part of SoftSERVE.
 *
 *   SoftSERVE is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   SoftSERVE is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with SoftSERVE.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * This file contains the difference structures arising from plus distributions tying together the Measurement functions FXi, plus distribution denominators 
 * and contributing remainder/numerator functions XCAi for the 2D integration at all eps-orders of the Rest-CA structure. 
 * Example: CAI2EM2 ('CA' structure, 'I'ntegration over '2' dimensions, at 'E'ps to the 'M'inus '2'nd order) contains various logs of FXi functions 
 * evaluated at z=0,c=0,x=0,s5=0, i.e. with all the deltas from +-distributions, multiplied with the correct XCA functions to arrive at eps^-2. 
 * Starting from eps^-4 (4 deltas) we need 2 eps powers, which we get from XCA2 (which comes with eps^2) and no Logs, Log^1 terms multiplied with XCA1 (eps^1 each), 
 * or Log^2 terms multiplied with XCA0 (eps^2 from the Logs) 
 */

#include <2IntCA.h>
#include <Auxiliaries.h>
#include <cmath>
#include <XFactorCA.h>
#include <Input_Common.h>

using namespace std;

double CAI2EM4(double w, double s6) {
	return -(pow(mexp,-1)*pow(n,-1)*XCA0(w,s6,0,0,0))/16.;
}

double CAI2EM3(double w,double s6) {
	return -(pow(mexp,-1)*pow(n,-1)*((log(FA1(v(w),tl(s6),0,0,0,0)) + log(FA2(v(w),tl(s6),0,0,0,0)) + log(FB1(v(w),tl(s6),0,0,0,0)) + log(FB2(v(w),tl(s6),0,0,0,0)))*XCA0(w,s6,0,0,0) + XCA1(w,s6,0,0,0)))/
			   16.;
}

double CAI2EM2(double w,double s6) {
	return -(pow(mexp,-1)*pow(n,-1)*(2*(pow(log(FA1(v(w),tl(s6),0,0,0,0)),2) + pow(log(FA2(v(w),tl(s6),0,0,0,0)),2) + pow(log(FB1(v(w),tl(s6),0,0,0,0)),2) + pow(log(FB2(v(w),tl(s6),0,0,0,0)),2))*
	         XCA0(w,s6,0,0,0) + (log(FA1(v(w),tl(s6),0,0,0,0)) + log(FA2(v(w),tl(s6),0,0,0,0)) + log(FB1(v(w),tl(s6),0,0,0,0)) + log(FB2(v(w),tl(s6),0,0,0,0)))*XCA1(w,s6,0,0,0) + XCA2(w,s6,0,0,0)
	        ))/16.;
}

double CAI2EM1(double w,double s6) {
	return -(pow(mexp,-1)*pow(n,-1)*(8*(pow(log(FA1(v(w),tl(s6),0,0,0,0)),3) + pow(log(FA2(v(w),tl(s6),0,0,0,0)),3) + pow(log(FB1(v(w),tl(s6),0,0,0,0)),3) + pow(log(FB2(v(w),tl(s6),0,0,0,0)),3))*
	         XCA0(w,s6,0,0,0) + 3*(2*(pow(log(FA1(v(w),tl(s6),0,0,0,0)),2) + pow(log(FA2(v(w),tl(s6),0,0,0,0)),2) + pow(log(FB1(v(w),tl(s6),0,0,0,0)),2) + pow(log(FB2(v(w),tl(s6),0,0,0,0)),2))*
	            XCA1(w,s6,0,0,0) + (log(FA1(v(w),tl(s6),0,0,0,0)) + log(FA2(v(w),tl(s6),0,0,0,0)) + log(FB1(v(w),tl(s6),0,0,0,0)) + log(FB2(v(w),tl(s6),0,0,0,0)))*XCA2(w,s6,0,0,0) +
	           XCA3(w,s6,0,0,0))))/48.;
}

double CAI2E0(double w,double s6) {
	return -(pow(mexp,-1)*pow(n,-1)*(8*(pow(log(FA1(v(w),tl(s6),0,0,0,0)),4) + pow(log(FA2(v(w),tl(s6),0,0,0,0)),4) + pow(log(FB1(v(w),tl(s6),0,0,0,0)),4) + pow(log(FB2(v(w),tl(s6),0,0,0,0)),4))*
	         XCA0(w,s6,0,0,0) + 8*(pow(log(FA1(v(w),tl(s6),0,0,0,0)),3) + pow(log(FA2(v(w),tl(s6),0,0,0,0)),3) + pow(log(FB1(v(w),tl(s6),0,0,0,0)),3) + pow(log(FB2(v(w),tl(s6),0,0,0,0)),3))*
	         XCA1(w,s6,0,0,0) + 3*(2*(pow(log(FA1(v(w),tl(s6),0,0,0,0)),2) + pow(log(FA2(v(w),tl(s6),0,0,0,0)),2) + pow(log(FB1(v(w),tl(s6),0,0,0,0)),2) + pow(log(FB2(v(w),tl(s6),0,0,0,0)),2))*
	            XCA2(w,s6,0,0,0) + (log(FA1(v(w),tl(s6),0,0,0,0)) + log(FA2(v(w),tl(s6),0,0,0,0)) + log(FB1(v(w),tl(s6),0,0,0,0)) + log(FB2(v(w),tl(s6),0,0,0,0)))*XCA3(w,s6,0,0,0) +
	           XCA4(w,s6,0,0,0))))/48.;
}
