/*This file is part of SoftSERVE.
 *
 *   SoftSERVE is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   SoftSERVE is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with SoftSERVE.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * This file contains the difference structures arising from plus distributions tying together the Measurement functions FXi, plus distribution denominators 
 * and contributing remainder/numerator functions XCAi for the 4D integration at all eps-orders of the Rest-CA structure. 
 * Example: CAI4EM1tu ('CA' structure, 'I'ntegration over '4' dimensions, at 'E'ps to the 'M'inus '1'st order, third and fourth integration over 't'5 and 'u' or their rescaled versions) 
 * contains various differences of logs of FXi functions evaluated at c=0,x=0, i.e. with deltas from all +-distributions except the t5/s5 and u/z ones, multiplied 
 * with the correct XCA functions to arrive at eps^-1. 
 * Starting from eps^-2 (2 deltas) we need 1 eps power, which we get from XCA1 (which comes with eps^1) and no Logs, or Log^1 terms multiplied with XCA0 (eps^1 from the logs)
 */

#include <4IntCA.h>
#include <Auxiliaries.h>
#include <cmath>
#include <XFactorCA.h>
#include <Input_Common.h>


//third and forth integration over y (x after rescaling), t5 (s5 after rescaling)

double CAI4EM2yt(double w,double s6,double x,double s5) {
	return 0;
}

double CAI4EM1yt(double w,double s6,double x,double s5) {
	return (pow(s5,-1)*pow(x,-1)*((log(FA1(v(w),tl(s6),0,0,0,0)) - log(FA1(v(w),tl(s6),y(x),0,0,0)) + log(FA2(v(w),tl(s6),0,0,0,0)) - log(FA2(v(w),tl(s6),y(x),0,0,0)) + log(FB1(v(w),tl(s6),0,0,0,0)) -
	          log(FB1(v(w),tl(s6),y(x),0,0,0)) + log(FB2(v(w),tl(s6),0,0,0,0)) - log(FB2(v(w),tl(s6),y(x),0,0,0)))*XCA0(w,s6,0,0,0) +
	       (-log(FA1(v(w),tl(s6),0,t5(s5),0,0)) + log(FA1(v(w),tl(s6),y(x),t5(s5),0,0)) - log(FA2(v(w),tl(s6),0,t5(s5),0,0)) + log(FA2(v(w),tl(s6),y(x),t5(s5),0,0)) -
	          log(FB1(v(w),tl(s6),0,t5(s5),0,0)) + log(FB1(v(w),tl(s6),y(x),t5(s5),0,0)) - log(FB2(v(w),tl(s6),0,t5(s5),0,0)) + log(FB2(v(w),tl(s6),y(x),t5(s5),0,0)))*XCA0(w,s6,s5,0,0)))/4.;
}

double CAI4E0yt(double w,double s6,double x,double s5) {
	return (pow(s5,-1)*pow(x,-1)*(-2*(log(s5)*(log(FA1(v(w),tl(s6),0,0,0,0)) - log(FA1(v(w),tl(s6),y(x),0,0,0)) + log(FA2(v(w),tl(s6),0,0,0,0)) - log(FA2(v(w),tl(s6),y(x),0,0,0)) +
            log(FB1(v(w),tl(s6),0,0,0,0)) - log(FB1(v(w),tl(s6),y(x),0,0,0)) + log(FB2(v(w),tl(s6),0,0,0,0)) - log(FB2(v(w),tl(s6),y(x),0,0,0))) +
         mexp*n*log(x)*(-log(FA1(v(w),tl(s6),0,0,0,0)) + log(FA1(v(w),tl(s6),y(x),0,0,0)) - log(FA2(v(w),tl(s6),0,0,0,0)) + log(FA2(v(w),tl(s6),y(x),0,0,0)) - log(FB1(v(w),tl(s6),0,0,0,0)) +
            log(FB1(v(w),tl(s6),y(x),0,0,0)) - log(FB2(v(w),tl(s6),0,0,0,0)) + log(FB2(v(w),tl(s6),y(x),0,0,0))) - pow(log(FA1(v(w),tl(s6),0,0,0,0)),2) +
         pow(log(FA1(v(w),tl(s6),y(x),0,0,0)),2) - pow(log(FA2(v(w),tl(s6),0,0,0,0)),2) + pow(log(FA2(v(w),tl(s6),y(x),0,0,0)),2) - pow(log(FB1(v(w),tl(s6),0,0,0,0)),2) +
         pow(log(FB1(v(w),tl(s6),y(x),0,0,0)),2) - pow(log(FB2(v(w),tl(s6),0,0,0,0)),2) + pow(log(FB2(v(w),tl(s6),y(x),0,0,0)),2))*XCA0(w,s6,0,0,0) +
      2*(log(s5)*(log(FA1(v(w),tl(s6),0,t5(s5),0,0)) - log(FA1(v(w),tl(s6),y(x),t5(s5),0,0)) + log(FA2(v(w),tl(s6),0,t5(s5),0,0)) - log(FA2(v(w),tl(s6),y(x),t5(s5),0,0)) +
            log(FB1(v(w),tl(s6),0,t5(s5),0,0)) - log(FB1(v(w),tl(s6),y(x),t5(s5),0,0)) + log(FB2(v(w),tl(s6),0,t5(s5),0,0)) - log(FB2(v(w),tl(s6),y(x),t5(s5),0,0))) +
         mexp*n*log(x)*(-log(FA1(v(w),tl(s6),0,t5(s5),0,0)) + log(FA1(v(w),tl(s6),y(x),t5(s5),0,0)) - log(FA2(v(w),tl(s6),0,t5(s5),0,0)) + log(FA2(v(w),tl(s6),y(x),t5(s5),0,0)) -
            log(FB1(v(w),tl(s6),0,t5(s5),0,0)) + log(FB1(v(w),tl(s6),y(x),t5(s5),0,0)) - log(FB2(v(w),tl(s6),0,t5(s5),0,0)) + log(FB2(v(w),tl(s6),y(x),t5(s5),0,0))) -
         pow(log(FA1(v(w),tl(s6),0,t5(s5),0,0)),2) + pow(log(FA1(v(w),tl(s6),y(x),t5(s5),0,0)),2) - pow(log(FA2(v(w),tl(s6),0,t5(s5),0,0)),2) +
         pow(log(FA2(v(w),tl(s6),y(x),t5(s5),0,0)),2) - pow(log(FB1(v(w),tl(s6),0,t5(s5),0,0)),2) + pow(log(FB1(v(w),tl(s6),y(x),t5(s5),0,0)),2) -
         pow(log(FB2(v(w),tl(s6),0,t5(s5),0,0)),2) + pow(log(FB2(v(w),tl(s6),y(x),t5(s5),0,0)),2))*XCA0(w,s6,s5,0,0) + log(FA1(v(w),tl(s6),0,0,0,0))*XCA1(w,s6,0,0,0) -
      log(FA1(v(w),tl(s6),y(x),0,0,0))*XCA1(w,s6,0,0,0) + log(FA2(v(w),tl(s6),0,0,0,0))*XCA1(w,s6,0,0,0) - log(FA2(v(w),tl(s6),y(x),0,0,0))*XCA1(w,s6,0,0,0) +
      log(FB1(v(w),tl(s6),0,0,0,0))*XCA1(w,s6,0,0,0) - log(FB1(v(w),tl(s6),y(x),0,0,0))*XCA1(w,s6,0,0,0) + log(FB2(v(w),tl(s6),0,0,0,0))*XCA1(w,s6,0,0,0) -
      log(FB2(v(w),tl(s6),y(x),0,0,0))*XCA1(w,s6,0,0,0) - log(FA1(v(w),tl(s6),0,t5(s5),0,0))*XCA1(w,s6,s5,0,0) + log(FA1(v(w),tl(s6),y(x),t5(s5),0,0))*XCA1(w,s6,s5,0,0) -
      log(FA2(v(w),tl(s6),0,t5(s5),0,0))*XCA1(w,s6,s5,0,0) + log(FA2(v(w),tl(s6),y(x),t5(s5),0,0))*XCA1(w,s6,s5,0,0) - log(FB1(v(w),tl(s6),0,t5(s5),0,0))*XCA1(w,s6,s5,0,0) +
      log(FB1(v(w),tl(s6),y(x),t5(s5),0,0))*XCA1(w,s6,s5,0,0) - log(FB2(v(w),tl(s6),0,t5(s5),0,0))*XCA1(w,s6,s5,0,0) + log(FB2(v(w),tl(s6),y(x),t5(s5),0,0))*XCA1(w,s6,s5,0,0)))/4.;
}

//third and forth integration over y (x after rescaling), u (z after rescaling)

double CAI4EM2yu(double w,double s6,double x,double z) {
	return 0;
}

double CAI4EM1yu(double w,double s6,double x,double z) {
	return (pow(x,-1)*pow(z,-1)*((log(FA1(v(w),tl(s6),0,0,0,0)) - log(FA1(v(w),tl(s6),y(x),0,0,0)) + log(FA2(v(w),tl(s6),0,0,0,0)) - log(FA2(v(w),tl(s6),y(x),0,0,0)) + log(FB1(v(w),tl(s6),0,0,0,0)) -
	          log(FB1(v(w),tl(s6),y(x),0,0,0)) + log(FB2(v(w),tl(s6),0,0,0,0)) - log(FB2(v(w),tl(s6),y(x),0,0,0)))*XCA0(w,s6,0,0,0) +
	       (-log(FA1(v(w),tl(s6),0,0,u(z),0)) + log(FA1(v(w),tl(s6),y(x),0,u(z),0)) - log(FA2(v(w),tl(s6),0,0,u(z),0)) + log(FA2(v(w),tl(s6),y(x),0,u(z),0)) - log(FB1(v(w),tl(s6),0,0,u(z),0)) +
	          log(FB1(v(w),tl(s6),y(x),0,u(z),0)) - log(FB2(v(w),tl(s6),0,0,u(z),0)) + log(FB2(v(w),tl(s6),y(x),0,u(z),0)))*XCA0(w,s6,0,z,0)))/2.;
}

double CAI4E0yu(double w,double s6,double x,double z) {
	return (pow(x,-1)*pow(z,-1)*(2*(mexp*n*log(x)*(log(FA1(v(w),tl(s6),0,0,0,0)) - log(FA1(v(w),tl(s6),y(x),0,0,0)) + log(FA2(v(w),tl(s6),0,0,0,0)) - log(FA2(v(w),tl(s6),y(x),0,0,0)) +
            log(FB1(v(w),tl(s6),0,0,0,0)) - log(FB1(v(w),tl(s6),y(x),0,0,0)) + log(FB2(v(w),tl(s6),0,0,0,0)) - log(FB2(v(w),tl(s6),y(x),0,0,0))) -
         2*log(z)*(log(FA1(v(w),tl(s6),0,0,0,0)) - log(FA1(v(w),tl(s6),y(x),0,0,0)) + log(FA2(v(w),tl(s6),0,0,0,0)) - log(FA2(v(w),tl(s6),y(x),0,0,0)) + log(FB1(v(w),tl(s6),0,0,0,0)) -
            log(FB1(v(w),tl(s6),y(x),0,0,0)) + log(FB2(v(w),tl(s6),0,0,0,0)) - log(FB2(v(w),tl(s6),y(x),0,0,0))) + pow(log(FA1(v(w),tl(s6),0,0,0,0)),2) -
         pow(log(FA1(v(w),tl(s6),y(x),0,0,0)),2) + pow(log(FA2(v(w),tl(s6),0,0,0,0)),2) - pow(log(FA2(v(w),tl(s6),y(x),0,0,0)),2) + pow(log(FB1(v(w),tl(s6),0,0,0,0)),2) -
         pow(log(FB1(v(w),tl(s6),y(x),0,0,0)),2) + pow(log(FB2(v(w),tl(s6),0,0,0,0)),2) - pow(log(FB2(v(w),tl(s6),y(x),0,0,0)),2))*XCA0(w,s6,0,0,0) -
      2*(mexp*n*log(x)*(log(FA1(v(w),tl(s6),0,0,u(z),0)) - log(FA1(v(w),tl(s6),y(x),0,u(z),0)) + log(FA2(v(w),tl(s6),0,0,u(z),0)) - log(FA2(v(w),tl(s6),y(x),0,u(z),0)) +
            log(FB1(v(w),tl(s6),0,0,u(z),0)) - log(FB1(v(w),tl(s6),y(x),0,u(z),0)) + log(FB2(v(w),tl(s6),0,0,u(z),0)) - log(FB2(v(w),tl(s6),y(x),0,u(z),0))) -
         2*log(z)*(log(FA1(v(w),tl(s6),0,0,u(z),0)) - log(FA1(v(w),tl(s6),y(x),0,u(z),0)) + log(FA2(v(w),tl(s6),0,0,u(z),0)) - log(FA2(v(w),tl(s6),y(x),0,u(z),0)) +
            log(FB1(v(w),tl(s6),0,0,u(z),0)) - log(FB1(v(w),tl(s6),y(x),0,u(z),0)) + log(FB2(v(w),tl(s6),0,0,u(z),0)) - log(FB2(v(w),tl(s6),y(x),0,u(z),0))) +
         pow(log(FA1(v(w),tl(s6),0,0,u(z),0)),2) - pow(log(FA1(v(w),tl(s6),y(x),0,u(z),0)),2) + pow(log(FA2(v(w),tl(s6),0,0,u(z),0)),2) - pow(log(FA2(v(w),tl(s6),y(x),0,u(z),0)),2) +
         pow(log(FB1(v(w),tl(s6),0,0,u(z),0)),2) - pow(log(FB1(v(w),tl(s6),y(x),0,u(z),0)),2) + pow(log(FB2(v(w),tl(s6),0,0,u(z),0)),2) - pow(log(FB2(v(w),tl(s6),y(x),0,u(z),0)),2))*
       XCA0(w,s6,0,z,0) + log(FA1(v(w),tl(s6),0,0,0,0))*XCA1(w,s6,0,0,0) - log(FA1(v(w),tl(s6),y(x),0,0,0))*XCA1(w,s6,0,0,0) + log(FA2(v(w),tl(s6),0,0,0,0))*XCA1(w,s6,0,0,0) -
      log(FA2(v(w),tl(s6),y(x),0,0,0))*XCA1(w,s6,0,0,0) + log(FB1(v(w),tl(s6),0,0,0,0))*XCA1(w,s6,0,0,0) - log(FB1(v(w),tl(s6),y(x),0,0,0))*XCA1(w,s6,0,0,0) +
      log(FB2(v(w),tl(s6),0,0,0,0))*XCA1(w,s6,0,0,0) - log(FB2(v(w),tl(s6),y(x),0,0,0))*XCA1(w,s6,0,0,0) - log(FA1(v(w),tl(s6),0,0,u(z),0))*XCA1(w,s6,0,z,0) +
      log(FA1(v(w),tl(s6),y(x),0,u(z),0))*XCA1(w,s6,0,z,0) - log(FA2(v(w),tl(s6),0,0,u(z),0))*XCA1(w,s6,0,z,0) + log(FA2(v(w),tl(s6),y(x),0,u(z),0))*XCA1(w,s6,0,z,0) -
      log(FB1(v(w),tl(s6),0,0,u(z),0))*XCA1(w,s6,0,z,0) + log(FB1(v(w),tl(s6),y(x),0,u(z),0))*XCA1(w,s6,0,z,0) - log(FB2(v(w),tl(s6),0,0,u(z),0))*XCA1(w,s6,0,z,0) +
      log(FB2(v(w),tl(s6),y(x),0,u(z),0))*XCA1(w,s6,0,z,0)))/2.;
}


//third and forth integration over y (x after rescaling), b (c after rescaling)

double CAI4EM2yb(double w,double s6,double x,double c) {
	return 0;
}

double CAI4EM1yb(double w,double s6,double x,double c) {
	return (pow(c,-1)*pow(x,-1)*((log(FA1(v(w),tl(s6),0,0,0,0)) - log(FA1(v(w),tl(s6),y(x),0,0,0)) + log(FA2(v(w),tl(s6),0,0,0,0)) - log(FA2(v(w),tl(s6),y(x),0,0,0)) + log(FB1(v(w),tl(s6),0,0,0,0)) -
	          log(FB1(v(w),tl(s6),y(x),0,0,0)) + log(FB2(v(w),tl(s6),0,0,0,0)) - log(FB2(v(w),tl(s6),y(x),0,0,0)))*XCA0(w,s6,0,0,0) +
	       (-log(FA1(v(w),tl(s6),0,0,0,b(c))) + log(FA1(v(w),tl(s6),y(x),0,0,b(c))) - log(FA2(v(w),tl(s6),0,0,0,b(c))) + log(FA2(v(w),tl(s6),y(x),0,0,b(c))) - log(FB1(v(w),tl(s6),0,0,0,b(c))) +
	          log(FB1(v(w),tl(s6),y(x),0,0,b(c))) - log(FB2(v(w),tl(s6),0,0,0,b(c))) + log(FB2(v(w),tl(s6),y(x),0,0,b(c))))*XCA0(w,s6,0,0,c)))/2.;
}

double CAI4E0yb(double w,double s6,double x,double c) {
	return (pow(c,-1)*pow(x,-1)*(-2*(2*log(c)*(log(FA1(v(w),tl(s6),0,0,0,0)) - log(FA1(v(w),tl(s6),y(x),0,0,0)) + log(FA2(v(w),tl(s6),0,0,0,0)) - log(FA2(v(w),tl(s6),y(x),0,0,0)) +
            log(FB1(v(w),tl(s6),0,0,0,0)) - log(FB1(v(w),tl(s6),y(x),0,0,0)) + log(FB2(v(w),tl(s6),0,0,0,0)) - log(FB2(v(w),tl(s6),y(x),0,0,0))) +
         mexp*n*log(x)*(-log(FA1(v(w),tl(s6),0,0,0,0)) + log(FA1(v(w),tl(s6),y(x),0,0,0)) - log(FA2(v(w),tl(s6),0,0,0,0)) + log(FA2(v(w),tl(s6),y(x),0,0,0)) - log(FB1(v(w),tl(s6),0,0,0,0)) +
            log(FB1(v(w),tl(s6),y(x),0,0,0)) - log(FB2(v(w),tl(s6),0,0,0,0)) + log(FB2(v(w),tl(s6),y(x),0,0,0))) - pow(log(FA1(v(w),tl(s6),0,0,0,0)),2) +
         pow(log(FA1(v(w),tl(s6),y(x),0,0,0)),2) - pow(log(FA2(v(w),tl(s6),0,0,0,0)),2) + pow(log(FA2(v(w),tl(s6),y(x),0,0,0)),2) - pow(log(FB1(v(w),tl(s6),0,0,0,0)),2) +
         pow(log(FB1(v(w),tl(s6),y(x),0,0,0)),2) - pow(log(FB2(v(w),tl(s6),0,0,0,0)),2) + pow(log(FB2(v(w),tl(s6),y(x),0,0,0)),2))*XCA0(w,s6,0,0,0) +
      2*(2*log(c)*(log(FA1(v(w),tl(s6),0,0,0,b(c))) - log(FA1(v(w),tl(s6),y(x),0,0,b(c))) + log(FA2(v(w),tl(s6),0,0,0,b(c))) - log(FA2(v(w),tl(s6),y(x),0,0,b(c))) +
            log(FB1(v(w),tl(s6),0,0,0,b(c))) - log(FB1(v(w),tl(s6),y(x),0,0,b(c))) + log(FB2(v(w),tl(s6),0,0,0,b(c))) - log(FB2(v(w),tl(s6),y(x),0,0,b(c)))) +
         mexp*n*log(x)*(-log(FA1(v(w),tl(s6),0,0,0,b(c))) + log(FA1(v(w),tl(s6),y(x),0,0,b(c))) - log(FA2(v(w),tl(s6),0,0,0,b(c))) + log(FA2(v(w),tl(s6),y(x),0,0,b(c))) -
            log(FB1(v(w),tl(s6),0,0,0,b(c))) + log(FB1(v(w),tl(s6),y(x),0,0,b(c))) - log(FB2(v(w),tl(s6),0,0,0,b(c))) + log(FB2(v(w),tl(s6),y(x),0,0,b(c)))) -
         pow(log(FA1(v(w),tl(s6),0,0,0,b(c))),2) + pow(log(FA1(v(w),tl(s6),y(x),0,0,b(c))),2) - pow(log(FA2(v(w),tl(s6),0,0,0,b(c))),2) + pow(log(FA2(v(w),tl(s6),y(x),0,0,b(c))),2) -
         pow(log(FB1(v(w),tl(s6),0,0,0,b(c))),2) + pow(log(FB1(v(w),tl(s6),y(x),0,0,b(c))),2) - pow(log(FB2(v(w),tl(s6),0,0,0,b(c))),2) + pow(log(FB2(v(w),tl(s6),y(x),0,0,b(c))),2))*
       XCA0(w,s6,0,0,c) + log(FA1(v(w),tl(s6),0,0,0,0))*XCA1(w,s6,0,0,0) - log(FA1(v(w),tl(s6),y(x),0,0,0))*XCA1(w,s6,0,0,0) + log(FA2(v(w),tl(s6),0,0,0,0))*XCA1(w,s6,0,0,0) -
      log(FA2(v(w),tl(s6),y(x),0,0,0))*XCA1(w,s6,0,0,0) + log(FB1(v(w),tl(s6),0,0,0,0))*XCA1(w,s6,0,0,0) - log(FB1(v(w),tl(s6),y(x),0,0,0))*XCA1(w,s6,0,0,0) +
      log(FB2(v(w),tl(s6),0,0,0,0))*XCA1(w,s6,0,0,0) - log(FB2(v(w),tl(s6),y(x),0,0,0))*XCA1(w,s6,0,0,0) - log(FA1(v(w),tl(s6),0,0,0,b(c)))*XCA1(w,s6,0,0,c) +
      log(FA1(v(w),tl(s6),y(x),0,0,b(c)))*XCA1(w,s6,0,0,c) - log(FA2(v(w),tl(s6),0,0,0,b(c)))*XCA1(w,s6,0,0,c) + log(FA2(v(w),tl(s6),y(x),0,0,b(c)))*XCA1(w,s6,0,0,c) -
      log(FB1(v(w),tl(s6),0,0,0,b(c)))*XCA1(w,s6,0,0,c) + log(FB1(v(w),tl(s6),y(x),0,0,b(c)))*XCA1(w,s6,0,0,c) - log(FB2(v(w),tl(s6),0,0,0,b(c)))*XCA1(w,s6,0,0,c) +
      log(FB2(v(w),tl(s6),y(x),0,0,b(c)))*XCA1(w,s6,0,0,c)))/2.;
}


//third and forth integration over t5 (s5 after rescaling), z (z after rescaling)

double CAI4EM2tu(double w,double s6,double s5,double z) {
	return (pow(mexp,-1)*pow(n,-1)*pow(s5,-1)*pow(z,-1)*(-XCA0(w,s6,0,0,0) + XCA0(w,s6,0,z,0) + XCA0(w,s6,s5,0,0) - XCA0(w,s6,s5,z,0)))/2.;
}

double CAI4EM1tu(double w,double s6,double s5,double z) {
	return (pow(mexp,-1)*pow(n,-1)*pow(s5,-1)*pow(z,-1)*((2*log(s5) + 4*log(z) - log(FA1(v(w),tl(s6),0,0,0,0)) - log(FA2(v(w),tl(s6),0,0,0,0)) - log(FB1(v(w),tl(s6),0,0,0,0)) -
	          log(FB2(v(w),tl(s6),0,0,0,0)))*XCA0(w,s6,0,0,0) + (-2*log(s5) - 4*log(z) + log(FA1(v(w),tl(s6),0,0,u(z),0)) + log(FA2(v(w),tl(s6),0,0,u(z),0)) + log(FB1(v(w),tl(s6),0,0,u(z),0)) +
	          log(FB2(v(w),tl(s6),0,0,u(z),0)))*XCA0(w,s6,0,z,0) - 2*log(s5)*XCA0(w,s6,s5,0,0) - 4*log(z)*XCA0(w,s6,s5,0,0) + log(FA1(v(w),tl(s6),0,t5(s5),0,0))*XCA0(w,s6,s5,0,0) +
	       log(FA2(v(w),tl(s6),0,t5(s5),0,0))*XCA0(w,s6,s5,0,0) + log(FB1(v(w),tl(s6),0,t5(s5),0,0))*XCA0(w,s6,s5,0,0) + log(FB2(v(w),tl(s6),0,t5(s5),0,0))*XCA0(w,s6,s5,0,0) +
	       2*log(s5)*XCA0(w,s6,s5,z,0) + 4*log(z)*XCA0(w,s6,s5,z,0) - log(FA1(v(w),tl(s6),0,t5(s5),u(z),0))*XCA0(w,s6,s5,z,0) - log(FA2(v(w),tl(s6),0,t5(s5),u(z),0))*XCA0(w,s6,s5,z,0) -
	       log(FB1(v(w),tl(s6),0,t5(s5),u(z),0))*XCA0(w,s6,s5,z,0) - log(FB2(v(w),tl(s6),0,t5(s5),u(z),0))*XCA0(w,s6,s5,z,0) - XCA1(w,s6,0,0,0) + XCA1(w,s6,0,z,0) + XCA1(w,s6,s5,0,0) -
	       XCA1(w,s6,s5,z,0)))/2.;
}

double CAI4E0tu(double w,double s6,double s5,double z) {
	return (pow(mexp,-1)*pow(n,-1)*pow(s5,-1)*pow(z,-1)*(-2*(log(s5)*(4*log(z) - log(FA1(v(w),tl(s6),0,0,0,0)) - log(FA2(v(w),tl(s6),0,0,0,0)) - log(FB1(v(w),tl(s6),0,0,0,0)) -
            log(FB2(v(w),tl(s6),0,0,0,0))) - 2*log(z)*(log(FA1(v(w),tl(s6),0,0,0,0)) + log(FA2(v(w),tl(s6),0,0,0,0)) + log(FB1(v(w),tl(s6),0,0,0,0)) + log(FB2(v(w),tl(s6),0,0,0,0))) +
         pow(log(s5),2) + 4*pow(log(z),2) + pow(log(FA1(v(w),tl(s6),0,0,0,0)),2) + pow(log(FA2(v(w),tl(s6),0,0,0,0)),2) + pow(log(FB1(v(w),tl(s6),0,0,0,0)),2) +
         pow(log(FB2(v(w),tl(s6),0,0,0,0)),2))*XCA0(w,s6,0,0,0) + 2*(log(s5)*
          (4*log(z) - log(FA1(v(w),tl(s6),0,0,u(z),0)) - log(FA2(v(w),tl(s6),0,0,u(z),0)) - log(FB1(v(w),tl(s6),0,0,u(z),0)) - log(FB2(v(w),tl(s6),0,0,u(z),0))) -
         2*log(z)*(log(FA1(v(w),tl(s6),0,0,u(z),0)) + log(FA2(v(w),tl(s6),0,0,u(z),0)) + log(FB1(v(w),tl(s6),0,0,u(z),0)) + log(FB2(v(w),tl(s6),0,0,u(z),0))) + pow(log(s5),2) +
         4*pow(log(z),2) + pow(log(FA1(v(w),tl(s6),0,0,u(z),0)),2) + pow(log(FA2(v(w),tl(s6),0,0,u(z),0)),2) + pow(log(FB1(v(w),tl(s6),0,0,u(z),0)),2) +
         pow(log(FB2(v(w),tl(s6),0,0,u(z),0)),2))*XCA0(w,s6,0,z,0) + 8*log(s5)*log(z)*XCA0(w,s6,s5,0,0) - 2*log(s5)*log(FA1(v(w),tl(s6),0,t5(s5),0,0))*XCA0(w,s6,s5,0,0) -
      4*log(z)*log(FA1(v(w),tl(s6),0,t5(s5),0,0))*XCA0(w,s6,s5,0,0) - 2*log(s5)*log(FA2(v(w),tl(s6),0,t5(s5),0,0))*XCA0(w,s6,s5,0,0) -
      4*log(z)*log(FA2(v(w),tl(s6),0,t5(s5),0,0))*XCA0(w,s6,s5,0,0) - 2*log(s5)*log(FB1(v(w),tl(s6),0,t5(s5),0,0))*XCA0(w,s6,s5,0,0) -
      4*log(z)*log(FB1(v(w),tl(s6),0,t5(s5),0,0))*XCA0(w,s6,s5,0,0) - 2*log(s5)*log(FB2(v(w),tl(s6),0,t5(s5),0,0))*XCA0(w,s6,s5,0,0) -
      4*log(z)*log(FB2(v(w),tl(s6),0,t5(s5),0,0))*XCA0(w,s6,s5,0,0) + 2*pow(log(s5),2)*XCA0(w,s6,s5,0,0) + 8*pow(log(z),2)*XCA0(w,s6,s5,0,0) +
      2*pow(log(FA1(v(w),tl(s6),0,t5(s5),0,0)),2)*XCA0(w,s6,s5,0,0) + 2*pow(log(FA2(v(w),tl(s6),0,t5(s5),0,0)),2)*XCA0(w,s6,s5,0,0) +
      2*pow(log(FB1(v(w),tl(s6),0,t5(s5),0,0)),2)*XCA0(w,s6,s5,0,0) + 2*pow(log(FB2(v(w),tl(s6),0,t5(s5),0,0)),2)*XCA0(w,s6,s5,0,0) - 8*log(s5)*log(z)*XCA0(w,s6,s5,z,0) +
      2*log(s5)*log(FA1(v(w),tl(s6),0,t5(s5),u(z),0))*XCA0(w,s6,s5,z,0) + 4*log(z)*log(FA1(v(w),tl(s6),0,t5(s5),u(z),0))*XCA0(w,s6,s5,z,0) +
      2*log(s5)*log(FA2(v(w),tl(s6),0,t5(s5),u(z),0))*XCA0(w,s6,s5,z,0) + 4*log(z)*log(FA2(v(w),tl(s6),0,t5(s5),u(z),0))*XCA0(w,s6,s5,z,0) +
      2*log(s5)*log(FB1(v(w),tl(s6),0,t5(s5),u(z),0))*XCA0(w,s6,s5,z,0) + 4*log(z)*log(FB1(v(w),tl(s6),0,t5(s5),u(z),0))*XCA0(w,s6,s5,z,0) +
      2*log(s5)*log(FB2(v(w),tl(s6),0,t5(s5),u(z),0))*XCA0(w,s6,s5,z,0) + 4*log(z)*log(FB2(v(w),tl(s6),0,t5(s5),u(z),0))*XCA0(w,s6,s5,z,0) - 2*pow(log(s5),2)*XCA0(w,s6,s5,z,0) -
      8*pow(log(z),2)*XCA0(w,s6,s5,z,0) - 2*pow(log(FA1(v(w),tl(s6),0,t5(s5),u(z),0)),2)*XCA0(w,s6,s5,z,0) - 2*pow(log(FA2(v(w),tl(s6),0,t5(s5),u(z),0)),2)*XCA0(w,s6,s5,z,0) -
      2*pow(log(FB1(v(w),tl(s6),0,t5(s5),u(z),0)),2)*XCA0(w,s6,s5,z,0) - 2*pow(log(FB2(v(w),tl(s6),0,t5(s5),u(z),0)),2)*XCA0(w,s6,s5,z,0) + 2*log(s5)*XCA1(w,s6,0,0,0) +
      4*log(z)*XCA1(w,s6,0,0,0) - log(FA1(v(w),tl(s6),0,0,0,0))*XCA1(w,s6,0,0,0) - log(FA2(v(w),tl(s6),0,0,0,0))*XCA1(w,s6,0,0,0) - log(FB1(v(w),tl(s6),0,0,0,0))*XCA1(w,s6,0,0,0) -
      log(FB2(v(w),tl(s6),0,0,0,0))*XCA1(w,s6,0,0,0) - 2*log(s5)*XCA1(w,s6,0,z,0) - 4*log(z)*XCA1(w,s6,0,z,0) + log(FA1(v(w),tl(s6),0,0,u(z),0))*XCA1(w,s6,0,z,0) +
      log(FA2(v(w),tl(s6),0,0,u(z),0))*XCA1(w,s6,0,z,0) + log(FB1(v(w),tl(s6),0,0,u(z),0))*XCA1(w,s6,0,z,0) + log(FB2(v(w),tl(s6),0,0,u(z),0))*XCA1(w,s6,0,z,0) -
      2*log(s5)*XCA1(w,s6,s5,0,0) - 4*log(z)*XCA1(w,s6,s5,0,0) + log(FA1(v(w),tl(s6),0,t5(s5),0,0))*XCA1(w,s6,s5,0,0) + log(FA2(v(w),tl(s6),0,t5(s5),0,0))*XCA1(w,s6,s5,0,0) +
      log(FB1(v(w),tl(s6),0,t5(s5),0,0))*XCA1(w,s6,s5,0,0) + log(FB2(v(w),tl(s6),0,t5(s5),0,0))*XCA1(w,s6,s5,0,0) + 2*log(s5)*XCA1(w,s6,s5,z,0) + 4*log(z)*XCA1(w,s6,s5,z,0) -
      log(FA1(v(w),tl(s6),0,t5(s5),u(z),0))*XCA1(w,s6,s5,z,0) - log(FA2(v(w),tl(s6),0,t5(s5),u(z),0))*XCA1(w,s6,s5,z,0) - log(FB1(v(w),tl(s6),0,t5(s5),u(z),0))*XCA1(w,s6,s5,z,0) -
      log(FB2(v(w),tl(s6),0,t5(s5),u(z),0))*XCA1(w,s6,s5,z,0) - XCA2(w,s6,0,0,0) + XCA2(w,s6,0,z,0) + XCA2(w,s6,s5,0,0) - XCA2(w,s6,s5,z,0)))/2.;
}


//third and forth integration over t5 (s5 after rescaling), b (c after rescaling)

double CAI4EM2tb(double w,double s6,double s5,double c) {
	return (pow(c,-1)*pow(mexp,-1)*pow(n,-1)*pow(s5,-1)*(-XCA0(w,s6,0,0,0) + XCA0(w,s6,0,0,c) + XCA0(w,s6,s5,0,0) - XCA0(w,s6,s5,0,c)))/2.;
}

double CAI4EM1tb(double w,double s6,double s5,double c) {
	return (pow(c,-1)*pow(mexp,-1)*pow(n,-1)*pow(s5,-1)*((4*log(c) + 2*log(s5) - log(FA1(v(w),tl(s6),0,0,0,0)) - log(FA2(v(w),tl(s6),0,0,0,0)) - log(FB1(v(w),tl(s6),0,0,0,0)) -
	          log(FB2(v(w),tl(s6),0,0,0,0)))*XCA0(w,s6,0,0,0) + (-4*log(c) - 2*log(s5) + log(FA1(v(w),tl(s6),0,0,0,b(c))) + log(FA2(v(w),tl(s6),0,0,0,b(c))) + log(FB1(v(w),tl(s6),0,0,0,b(c))) +
	          log(FB2(v(w),tl(s6),0,0,0,b(c))))*XCA0(w,s6,0,0,c) - 4*log(c)*XCA0(w,s6,s5,0,0) - 2*log(s5)*XCA0(w,s6,s5,0,0) + log(FA1(v(w),tl(s6),0,t5(s5),0,0))*XCA0(w,s6,s5,0,0) +
	       log(FA2(v(w),tl(s6),0,t5(s5),0,0))*XCA0(w,s6,s5,0,0) + log(FB1(v(w),tl(s6),0,t5(s5),0,0))*XCA0(w,s6,s5,0,0) + log(FB2(v(w),tl(s6),0,t5(s5),0,0))*XCA0(w,s6,s5,0,0) +
	       4*log(c)*XCA0(w,s6,s5,0,c) + 2*log(s5)*XCA0(w,s6,s5,0,c) - log(FA1(v(w),tl(s6),0,t5(s5),0,b(c)))*XCA0(w,s6,s5,0,c) - log(FA2(v(w),tl(s6),0,t5(s5),0,b(c)))*XCA0(w,s6,s5,0,c) -
	       log(FB1(v(w),tl(s6),0,t5(s5),0,b(c)))*XCA0(w,s6,s5,0,c) - log(FB2(v(w),tl(s6),0,t5(s5),0,b(c)))*XCA0(w,s6,s5,0,c) - XCA1(w,s6,0,0,0) + XCA1(w,s6,0,0,c) + XCA1(w,s6,s5,0,0) -
	       XCA1(w,s6,s5,0,c)))/2.;
}

double CAI4E0tb(double w,double s6,double s5,double c) {
	return (pow(c,-1)*pow(mexp,-1)*pow(n,-1)*pow(s5,-1)*(-2*(-(log(s5)*(log(FA1(v(w),tl(s6),0,0,0,0)) + log(FA2(v(w),tl(s6),0,0,0,0)) + log(FB1(v(w),tl(s6),0,0,0,0)) + log(FB2(v(w),tl(s6),0,0,0,0)))) +
	          log(c)*(4*log(s5) - 2*(log(FA1(v(w),tl(s6),0,0,0,0)) + log(FA2(v(w),tl(s6),0,0,0,0)) + log(FB1(v(w),tl(s6),0,0,0,0)) + log(FB2(v(w),tl(s6),0,0,0,0)))) + 4*pow(log(c),2) +
	          pow(log(s5),2) + pow(log(FA1(v(w),tl(s6),0,0,0,0)),2) + pow(log(FA2(v(w),tl(s6),0,0,0,0)),2) + pow(log(FB1(v(w),tl(s6),0,0,0,0)),2) + pow(log(FB2(v(w),tl(s6),0,0,0,0)),2))*
	        XCA0(w,s6,0,0,0) + 2*(-(log(s5)*(log(FA1(v(w),tl(s6),0,0,0,b(c))) + log(FA2(v(w),tl(s6),0,0,0,b(c))) + log(FB1(v(w),tl(s6),0,0,0,b(c))) + log(FB2(v(w),tl(s6),0,0,0,b(c))))) +
	          log(c)*(4*log(s5) - 2*(log(FA1(v(w),tl(s6),0,0,0,b(c))) + log(FA2(v(w),tl(s6),0,0,0,b(c))) + log(FB1(v(w),tl(s6),0,0,0,b(c))) + log(FB2(v(w),tl(s6),0,0,0,b(c))))) +
	          4*pow(log(c),2) + pow(log(s5),2) + pow(log(FA1(v(w),tl(s6),0,0,0,b(c))),2) + pow(log(FA2(v(w),tl(s6),0,0,0,b(c))),2) + pow(log(FB1(v(w),tl(s6),0,0,0,b(c))),2) +
	          pow(log(FB2(v(w),tl(s6),0,0,0,b(c))),2))*XCA0(w,s6,0,0,c) + 8*log(c)*log(s5)*XCA0(w,s6,s5,0,0) - 4*log(c)*log(FA1(v(w),tl(s6),0,t5(s5),0,0))*XCA0(w,s6,s5,0,0) -
	       2*log(s5)*log(FA1(v(w),tl(s6),0,t5(s5),0,0))*XCA0(w,s6,s5,0,0) - 4*log(c)*log(FA2(v(w),tl(s6),0,t5(s5),0,0))*XCA0(w,s6,s5,0,0) -
	       2*log(s5)*log(FA2(v(w),tl(s6),0,t5(s5),0,0))*XCA0(w,s6,s5,0,0) - 4*log(c)*log(FB1(v(w),tl(s6),0,t5(s5),0,0))*XCA0(w,s6,s5,0,0) -
	       2*log(s5)*log(FB1(v(w),tl(s6),0,t5(s5),0,0))*XCA0(w,s6,s5,0,0) - 4*log(c)*log(FB2(v(w),tl(s6),0,t5(s5),0,0))*XCA0(w,s6,s5,0,0) -
	       2*log(s5)*log(FB2(v(w),tl(s6),0,t5(s5),0,0))*XCA0(w,s6,s5,0,0) + 8*pow(log(c),2)*XCA0(w,s6,s5,0,0) + 2*pow(log(s5),2)*XCA0(w,s6,s5,0,0) +
	       2*pow(log(FA1(v(w),tl(s6),0,t5(s5),0,0)),2)*XCA0(w,s6,s5,0,0) + 2*pow(log(FA2(v(w),tl(s6),0,t5(s5),0,0)),2)*XCA0(w,s6,s5,0,0) +
	       2*pow(log(FB1(v(w),tl(s6),0,t5(s5),0,0)),2)*XCA0(w,s6,s5,0,0) + 2*pow(log(FB2(v(w),tl(s6),0,t5(s5),0,0)),2)*XCA0(w,s6,s5,0,0) - 8*log(c)*log(s5)*XCA0(w,s6,s5,0,c) +
	       4*log(c)*log(FA1(v(w),tl(s6),0,t5(s5),0,b(c)))*XCA0(w,s6,s5,0,c) + 2*log(s5)*log(FA1(v(w),tl(s6),0,t5(s5),0,b(c)))*XCA0(w,s6,s5,0,c) +
	       4*log(c)*log(FA2(v(w),tl(s6),0,t5(s5),0,b(c)))*XCA0(w,s6,s5,0,c) + 2*log(s5)*log(FA2(v(w),tl(s6),0,t5(s5),0,b(c)))*XCA0(w,s6,s5,0,c) +
	       4*log(c)*log(FB1(v(w),tl(s6),0,t5(s5),0,b(c)))*XCA0(w,s6,s5,0,c) + 2*log(s5)*log(FB1(v(w),tl(s6),0,t5(s5),0,b(c)))*XCA0(w,s6,s5,0,c) +
	       4*log(c)*log(FB2(v(w),tl(s6),0,t5(s5),0,b(c)))*XCA0(w,s6,s5,0,c) + 2*log(s5)*log(FB2(v(w),tl(s6),0,t5(s5),0,b(c)))*XCA0(w,s6,s5,0,c) - 8*pow(log(c),2)*XCA0(w,s6,s5,0,c) -
	       2*pow(log(s5),2)*XCA0(w,s6,s5,0,c) - 2*pow(log(FA1(v(w),tl(s6),0,t5(s5),0,b(c))),2)*XCA0(w,s6,s5,0,c) - 2*pow(log(FA2(v(w),tl(s6),0,t5(s5),0,b(c))),2)*XCA0(w,s6,s5,0,c) -
	       2*pow(log(FB1(v(w),tl(s6),0,t5(s5),0,b(c))),2)*XCA0(w,s6,s5,0,c) - 2*pow(log(FB2(v(w),tl(s6),0,t5(s5),0,b(c))),2)*XCA0(w,s6,s5,0,c) + 4*log(c)*XCA1(w,s6,0,0,0) +
	       2*log(s5)*XCA1(w,s6,0,0,0) - log(FA1(v(w),tl(s6),0,0,0,0))*XCA1(w,s6,0,0,0) - log(FA2(v(w),tl(s6),0,0,0,0))*XCA1(w,s6,0,0,0) - log(FB1(v(w),tl(s6),0,0,0,0))*XCA1(w,s6,0,0,0) -
	       log(FB2(v(w),tl(s6),0,0,0,0))*XCA1(w,s6,0,0,0) - 4*log(c)*XCA1(w,s6,0,0,c) - 2*log(s5)*XCA1(w,s6,0,0,c) + log(FA1(v(w),tl(s6),0,0,0,b(c)))*XCA1(w,s6,0,0,c) +
	       log(FA2(v(w),tl(s6),0,0,0,b(c)))*XCA1(w,s6,0,0,c) + log(FB1(v(w),tl(s6),0,0,0,b(c)))*XCA1(w,s6,0,0,c) + log(FB2(v(w),tl(s6),0,0,0,b(c)))*XCA1(w,s6,0,0,c) -
	       4*log(c)*XCA1(w,s6,s5,0,0) - 2*log(s5)*XCA1(w,s6,s5,0,0) + log(FA1(v(w),tl(s6),0,t5(s5),0,0))*XCA1(w,s6,s5,0,0) + log(FA2(v(w),tl(s6),0,t5(s5),0,0))*XCA1(w,s6,s5,0,0) +
	       log(FB1(v(w),tl(s6),0,t5(s5),0,0))*XCA1(w,s6,s5,0,0) + log(FB2(v(w),tl(s6),0,t5(s5),0,0))*XCA1(w,s6,s5,0,0) + 4*log(c)*XCA1(w,s6,s5,0,c) + 2*log(s5)*XCA1(w,s6,s5,0,c) -
	       log(FA1(v(w),tl(s6),0,t5(s5),0,b(c)))*XCA1(w,s6,s5,0,c) - log(FA2(v(w),tl(s6),0,t5(s5),0,b(c)))*XCA1(w,s6,s5,0,c) - log(FB1(v(w),tl(s6),0,t5(s5),0,b(c)))*XCA1(w,s6,s5,0,c) -
	       log(FB2(v(w),tl(s6),0,t5(s5),0,b(c)))*XCA1(w,s6,s5,0,c) - XCA2(w,s6,0,0,0) + XCA2(w,s6,0,0,c) + XCA2(w,s6,s5,0,0) - XCA2(w,s6,s5,0,c)))/2.;
}


//third and forth integration over u (z after rescaling), b (c after rescaling)

double CAI4EM2ub(double w,double s6,double z,double c) {
	return pow(c,-1)*pow(mexp,-1)*pow(n,-1)*pow(z,-1)*(-XCA0(w,s6,0,0,0) + XCA0(w,s6,0,0,c) + XCA0(w,s6,0,z,0) - XCA0(w,s6,0,z,c));
}

double CAI4EM1ub(double w,double s6,double z,double c) {
	return pow(c,-1)*pow(mexp,-1)*pow(n,-1)*pow(z,-1)*((4*log(c) + 4*log(z) - log(FA1(v(w),tl(s6),0,0,0,0)) - log(FA2(v(w),tl(s6),0,0,0,0)) - log(FB1(v(w),tl(s6),0,0,0,0)) - log(FB2(v(w),tl(s6),0,0,0,0)))*
		      XCA0(w,s6,0,0,0) + (-4*log(c) - 4*log(z) + log(FA1(v(w),tl(s6),0,0,0,b(c))) + log(FA2(v(w),tl(s6),0,0,0,b(c))) + log(FB1(v(w),tl(s6),0,0,0,b(c))) + log(FB2(v(w),tl(s6),0,0,0,b(c))))*
		      XCA0(w,s6,0,0,c) - 4*log(c)*XCA0(w,s6,0,z,0) - 4*log(z)*XCA0(w,s6,0,z,0) + log(FA1(v(w),tl(s6),0,0,u(z),0))*XCA0(w,s6,0,z,0) + log(FA2(v(w),tl(s6),0,0,u(z),0))*XCA0(w,s6,0,z,0) +
		     log(FB1(v(w),tl(s6),0,0,u(z),0))*XCA0(w,s6,0,z,0) + log(FB2(v(w),tl(s6),0,0,u(z),0))*XCA0(w,s6,0,z,0) + 4*log(c)*XCA0(w,s6,0,z,c) + 4*log(z)*XCA0(w,s6,0,z,c) -
		     log(FA1(v(w),tl(s6),0,0,u(z),b(c)))*XCA0(w,s6,0,z,c) - log(FA2(v(w),tl(s6),0,0,u(z),b(c)))*XCA0(w,s6,0,z,c) - log(FB1(v(w),tl(s6),0,0,u(z),b(c)))*XCA0(w,s6,0,z,c) -
		     log(FB2(v(w),tl(s6),0,0,u(z),b(c)))*XCA0(w,s6,0,z,c) - XCA1(w,s6,0,0,0) + XCA1(w,s6,0,0,c) + XCA1(w,s6,0,z,0) - XCA1(w,s6,0,z,c));
}

double CAI4E0ub(double w,double s6,double z,double c) {
	return pow(c,-1)*pow(mexp,-1)*pow(n,-1)*pow(z,-1)*(-2*(-2*log(z)*(log(FA1(v(w),tl(s6),0,0,0,0)) + log(FA2(v(w),tl(s6),0,0,0,0)) + log(FB1(v(w),tl(s6),0,0,0,0)) + log(FB2(v(w),tl(s6),0,0,0,0))) +
	        log(c)*(8*log(z) - 2*(log(FA1(v(w),tl(s6),0,0,0,0)) + log(FA2(v(w),tl(s6),0,0,0,0)) + log(FB1(v(w),tl(s6),0,0,0,0)) + log(FB2(v(w),tl(s6),0,0,0,0)))) + 4*pow(log(c),2) +
	        4*pow(log(z),2) + pow(log(FA1(v(w),tl(s6),0,0,0,0)),2) + pow(log(FA2(v(w),tl(s6),0,0,0,0)),2) + pow(log(FB1(v(w),tl(s6),0,0,0,0)),2) + pow(log(FB2(v(w),tl(s6),0,0,0,0)),2))*
	      XCA0(w,s6,0,0,0) + 2*(-2*log(z)*(log(FA1(v(w),tl(s6),0,0,0,b(c))) + log(FA2(v(w),tl(s6),0,0,0,b(c))) + log(FB1(v(w),tl(s6),0,0,0,b(c))) + log(FB2(v(w),tl(s6),0,0,0,b(c)))) +
	        log(c)*(8*log(z) - 2*(log(FA1(v(w),tl(s6),0,0,0,b(c))) + log(FA2(v(w),tl(s6),0,0,0,b(c))) + log(FB1(v(w),tl(s6),0,0,0,b(c))) + log(FB2(v(w),tl(s6),0,0,0,b(c))))) + 4*pow(log(c),2) +
	        4*pow(log(z),2) + pow(log(FA1(v(w),tl(s6),0,0,0,b(c))),2) + pow(log(FA2(v(w),tl(s6),0,0,0,b(c))),2) + pow(log(FB1(v(w),tl(s6),0,0,0,b(c))),2) + pow(log(FB2(v(w),tl(s6),0,0,0,b(c))),2)
	        )*XCA0(w,s6,0,0,c) + 16*log(c)*log(z)*XCA0(w,s6,0,z,0) - 4*log(c)*log(FA1(v(w),tl(s6),0,0,u(z),0))*XCA0(w,s6,0,z,0) - 4*log(z)*log(FA1(v(w),tl(s6),0,0,u(z),0))*XCA0(w,s6,0,z,0) -
	     4*log(c)*log(FA2(v(w),tl(s6),0,0,u(z),0))*XCA0(w,s6,0,z,0) - 4*log(z)*log(FA2(v(w),tl(s6),0,0,u(z),0))*XCA0(w,s6,0,z,0) - 4*log(c)*log(FB1(v(w),tl(s6),0,0,u(z),0))*XCA0(w,s6,0,z,0) -
	     4*log(z)*log(FB1(v(w),tl(s6),0,0,u(z),0))*XCA0(w,s6,0,z,0) - 4*log(c)*log(FB2(v(w),tl(s6),0,0,u(z),0))*XCA0(w,s6,0,z,0) - 4*log(z)*log(FB2(v(w),tl(s6),0,0,u(z),0))*XCA0(w,s6,0,z,0) +
	     8*pow(log(c),2)*XCA0(w,s6,0,z,0) + 8*pow(log(z),2)*XCA0(w,s6,0,z,0) + 2*pow(log(FA1(v(w),tl(s6),0,0,u(z),0)),2)*XCA0(w,s6,0,z,0) +
	     2*pow(log(FA2(v(w),tl(s6),0,0,u(z),0)),2)*XCA0(w,s6,0,z,0) + 2*pow(log(FB1(v(w),tl(s6),0,0,u(z),0)),2)*XCA0(w,s6,0,z,0) + 2*pow(log(FB2(v(w),tl(s6),0,0,u(z),0)),2)*XCA0(w,s6,0,z,0) -
	     16*log(c)*log(z)*XCA0(w,s6,0,z,c) + 4*log(c)*log(FA1(v(w),tl(s6),0,0,u(z),b(c)))*XCA0(w,s6,0,z,c) + 4*log(z)*log(FA1(v(w),tl(s6),0,0,u(z),b(c)))*XCA0(w,s6,0,z,c) +
	     4*log(c)*log(FA2(v(w),tl(s6),0,0,u(z),b(c)))*XCA0(w,s6,0,z,c) + 4*log(z)*log(FA2(v(w),tl(s6),0,0,u(z),b(c)))*XCA0(w,s6,0,z,c) +
	     4*log(c)*log(FB1(v(w),tl(s6),0,0,u(z),b(c)))*XCA0(w,s6,0,z,c) + 4*log(z)*log(FB1(v(w),tl(s6),0,0,u(z),b(c)))*XCA0(w,s6,0,z,c) +
	     4*log(c)*log(FB2(v(w),tl(s6),0,0,u(z),b(c)))*XCA0(w,s6,0,z,c) + 4*log(z)*log(FB2(v(w),tl(s6),0,0,u(z),b(c)))*XCA0(w,s6,0,z,c) - 8*pow(log(c),2)*XCA0(w,s6,0,z,c) -
	     8*pow(log(z),2)*XCA0(w,s6,0,z,c) - 2*pow(log(FA1(v(w),tl(s6),0,0,u(z),b(c))),2)*XCA0(w,s6,0,z,c) - 2*pow(log(FA2(v(w),tl(s6),0,0,u(z),b(c))),2)*XCA0(w,s6,0,z,c) -
	     2*pow(log(FB1(v(w),tl(s6),0,0,u(z),b(c))),2)*XCA0(w,s6,0,z,c) - 2*pow(log(FB2(v(w),tl(s6),0,0,u(z),b(c))),2)*XCA0(w,s6,0,z,c) + 4*log(c)*XCA1(w,s6,0,0,0) + 4*log(z)*XCA1(w,s6,0,0,0) -
	     log(FA1(v(w),tl(s6),0,0,0,0))*XCA1(w,s6,0,0,0) - log(FA2(v(w),tl(s6),0,0,0,0))*XCA1(w,s6,0,0,0) - log(FB1(v(w),tl(s6),0,0,0,0))*XCA1(w,s6,0,0,0) -
	     log(FB2(v(w),tl(s6),0,0,0,0))*XCA1(w,s6,0,0,0) - 4*log(c)*XCA1(w,s6,0,0,c) - 4*log(z)*XCA1(w,s6,0,0,c) + log(FA1(v(w),tl(s6),0,0,0,b(c)))*XCA1(w,s6,0,0,c) +
	     log(FA2(v(w),tl(s6),0,0,0,b(c)))*XCA1(w,s6,0,0,c) + log(FB1(v(w),tl(s6),0,0,0,b(c)))*XCA1(w,s6,0,0,c) + log(FB2(v(w),tl(s6),0,0,0,b(c)))*XCA1(w,s6,0,0,c) - 4*log(c)*XCA1(w,s6,0,z,0) -
	     4*log(z)*XCA1(w,s6,0,z,0) + log(FA1(v(w),tl(s6),0,0,u(z),0))*XCA1(w,s6,0,z,0) + log(FA2(v(w),tl(s6),0,0,u(z),0))*XCA1(w,s6,0,z,0) + log(FB1(v(w),tl(s6),0,0,u(z),0))*XCA1(w,s6,0,z,0) +
	     log(FB2(v(w),tl(s6),0,0,u(z),0))*XCA1(w,s6,0,z,0) + 4*log(c)*XCA1(w,s6,0,z,c) + 4*log(z)*XCA1(w,s6,0,z,c) - log(FA1(v(w),tl(s6),0,0,u(z),b(c)))*XCA1(w,s6,0,z,c) -
	     log(FA2(v(w),tl(s6),0,0,u(z),b(c)))*XCA1(w,s6,0,z,c) - log(FB1(v(w),tl(s6),0,0,u(z),b(c)))*XCA1(w,s6,0,z,c) - log(FB2(v(w),tl(s6),0,0,u(z),b(c)))*XCA1(w,s6,0,z,c) - XCA2(w,s6,0,0,0) +
	     XCA2(w,s6,0,0,c) + XCA2(w,s6,0,z,0) - XCA2(w,s6,0,z,c));
}
