/*This file is part of SoftSERVE.
 *
 *   SoftSERVE is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   SoftSERVE is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with SoftSERVE.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * This file contains the difference structures arising from plus distributions tying together the Measurement functions FXi, plus distribution denominators 
 * and contributing remainder/numerator functions XNfi for the 4D integration at all eps-orders of the nf structure. 
 * Example: NfI4EM1t ('Nf' structure, 'I'ntegration over '4' dimensions, at 'E'ps to the 'M'inus '1'st order, fourth integration over 't'5 or its rescaled version) 
 * contains logs of FXi functions evaluated at z=0,x=0, i.e. with the deltas from all +-distributions except the t5/s5 one, multiplied 
 * with the correct XNf functions to arrive at eps^-1. 
 * Starting from eps^-2 (2 deltas) we need 1 eps power, which we get from XNf1 (which comes with eps^1) and no Logs, or Log^1 terms multiplied with XNf0 (eps^1 from the logs)
 */

#include <4IntNF.h>
#include <Auxiliaries.h>
#include <cmath>
#include <XFactorNf.h>
#include <Input_Common.h>


//fourth integration is over y (x after rescaling)

double NfI4EM2y(double w,double s6,double x,double c) {
	return 0;
}

double NfI4EM1y(double w,double s6,double x,double c) {
	return ((-log(FA1(v(w),tl(s6),0,0,0,b(c))) + log(FA1(v(w),tl(s6),y(x),0,0,b(c))) - log(FA2(v(w),tl(s6),0,0,0,b(c))) + log(FA2(v(w),tl(s6),y(x),0,0,b(c))) - log(FB1(v(w),tl(s6),0,0,0,b(c))) +
		       log(FB1(v(w),tl(s6),y(x),0,0,b(c))) - log(FB2(v(w),tl(s6),0,0,0,b(c))) + log(FB2(v(w),tl(s6),y(x),0,0,b(c))))*pow(x,-1)*XNf0(w,s6,0,0,c))/2.;
}

double NfI4E0y(double w,double s6,double x,double c) {
	return (pow(x,-1)*(-2*(mexp*n*log(x)*(log(FA1(v(w),tl(s6),0,0,0,b(c))) - log(FA1(v(w),tl(s6),y(x),0,0,b(c))) + log(FA2(v(w),tl(s6),0,0,0,b(c))) - log(FA2(v(w),tl(s6),y(x),0,0,b(c))) +
            log(FB1(v(w),tl(s6),0,0,0,b(c))) - log(FB1(v(w),tl(s6),y(x),0,0,b(c))) + log(FB2(v(w),tl(s6),0,0,0,b(c))) - log(FB2(v(w),tl(s6),y(x),0,0,b(c)))) +
         pow(log(FA1(v(w),tl(s6),0,0,0,b(c))),2) - pow(log(FA1(v(w),tl(s6),y(x),0,0,b(c))),2) + pow(log(FA2(v(w),tl(s6),0,0,0,b(c))),2) - pow(log(FA2(v(w),tl(s6),y(x),0,0,b(c))),2) +
         pow(log(FB1(v(w),tl(s6),0,0,0,b(c))),2) - pow(log(FB1(v(w),tl(s6),y(x),0,0,b(c))),2) + pow(log(FB2(v(w),tl(s6),0,0,0,b(c))),2) - pow(log(FB2(v(w),tl(s6),y(x),0,0,b(c))),2))*
       XNf0(w,s6,0,0,c) + (-log(FA1(v(w),tl(s6),0,0,0,b(c))) + log(FA1(v(w),tl(s6),y(x),0,0,b(c))) - log(FA2(v(w),tl(s6),0,0,0,b(c))) + log(FA2(v(w),tl(s6),y(x),0,0,b(c))) -
         log(FB1(v(w),tl(s6),0,0,0,b(c))) + log(FB1(v(w),tl(s6),y(x),0,0,b(c))) - log(FB2(v(w),tl(s6),0,0,0,b(c))) + log(FB2(v(w),tl(s6),y(x),0,0,b(c))))*XNf1(w,s6,0,0,c)))/2.;
}

//fourth integration is over t5 (s5 after rescaling)

double NfI4EM2t(double w,double s6,double s5,double c) {
	return (pow(mexp,-1)*pow(n,-1)*pow(s5,-1)*(XNf0(w,s6,0,0,c) - XNf0(w,s6,s5,0,c)))/2.;
}

double NfI4EM1t(double w,double s6,double s5,double c) {
	return (pow(mexp,-1)*pow(n,-1)*pow(s5,-1)*((-2*log(s5) + log(FA1(v(w),tl(s6),0,0,0,b(c))) + log(FA2(v(w),tl(s6),0,0,0,b(c))) + log(FB1(v(w),tl(s6),0,0,0,b(c))) + log(FB2(v(w),tl(s6),0,0,0,b(c))))*
	        XNf0(w,s6,0,0,c) + (2*log(s5) - log(FA1(v(w),tl(s6),0,t5(s5),0,b(c))) - log(FA2(v(w),tl(s6),0,t5(s5),0,b(c))) - log(FB1(v(w),tl(s6),0,t5(s5),0,b(c))) -
	          log(FB2(v(w),tl(s6),0,t5(s5),0,b(c))))*XNf0(w,s6,s5,0,c) + XNf1(w,s6,0,0,c) - XNf1(w,s6,s5,0,c)))/2.;
}

double NfI4E0t(double w,double s6,double s5,double c) {
	return (pow(mexp,-1)*pow(n,-1)*pow(s5,-1)*(2*(-(log(s5)*(log(FA1(v(w),tl(s6),0,0,0,b(c))) + log(FA2(v(w),tl(s6),0,0,0,b(c))) + log(FB1(v(w),tl(s6),0,0,0,b(c))) + log(FB2(v(w),tl(s6),0,0,0,b(c))))) +
	          pow(log(s5),2) + pow(log(FA1(v(w),tl(s6),0,0,0,b(c))),2) + pow(log(FA2(v(w),tl(s6),0,0,0,b(c))),2) + pow(log(FB1(v(w),tl(s6),0,0,0,b(c))),2) +
	          pow(log(FB2(v(w),tl(s6),0,0,0,b(c))),2))*XNf0(w,s6,0,0,c) - 2*(-(log(s5)*
	             (log(FA1(v(w),tl(s6),0,t5(s5),0,b(c))) + log(FA2(v(w),tl(s6),0,t5(s5),0,b(c))) + log(FB1(v(w),tl(s6),0,t5(s5),0,b(c))) + log(FB2(v(w),tl(s6),0,t5(s5),0,b(c))))) +
	          pow(log(s5),2) + pow(log(FA1(v(w),tl(s6),0,t5(s5),0,b(c))),2) + pow(log(FA2(v(w),tl(s6),0,t5(s5),0,b(c))),2) + pow(log(FB1(v(w),tl(s6),0,t5(s5),0,b(c))),2) +
	          pow(log(FB2(v(w),tl(s6),0,t5(s5),0,b(c))),2))*XNf0(w,s6,s5,0,c) - 2*log(s5)*XNf1(w,s6,0,0,c) + log(FA1(v(w),tl(s6),0,0,0,b(c)))*XNf1(w,s6,0,0,c) +
	       log(FA2(v(w),tl(s6),0,0,0,b(c)))*XNf1(w,s6,0,0,c) + log(FB1(v(w),tl(s6),0,0,0,b(c)))*XNf1(w,s6,0,0,c) + log(FB2(v(w),tl(s6),0,0,0,b(c)))*XNf1(w,s6,0,0,c) +
	       2*log(s5)*XNf1(w,s6,s5,0,c) - log(FA1(v(w),tl(s6),0,t5(s5),0,b(c)))*XNf1(w,s6,s5,0,c) - log(FA2(v(w),tl(s6),0,t5(s5),0,b(c)))*XNf1(w,s6,s5,0,c) -
	       log(FB1(v(w),tl(s6),0,t5(s5),0,b(c)))*XNf1(w,s6,s5,0,c) - log(FB2(v(w),tl(s6),0,t5(s5),0,b(c)))*XNf1(w,s6,s5,0,c) + XNf2(w,s6,0,0,c) - XNf2(w,s6,s5,0,c)))/2.;
}

//fourth integration is over u (z after rescaling)

double NfI4EM2u(double w,double s6,double z,double c) {
	return pow(mexp,-1)*pow(n,-1)*pow(z,-1)*(XNf0(w,s6,0,0,c) - XNf0(w,s6,0,z,c));
}

double NfI4EM1u(double w,double s6,double z,double c) {
	return pow(mexp,-1)*pow(n,-1)*pow(z,-1)*((-4*log(z) + log(FA1(v(w),tl(s6),0,0,0,b(c))) + log(FA2(v(w),tl(s6),0,0,0,b(c))) + log(FB1(v(w),tl(s6),0,0,0,b(c))) + log(FB2(v(w),tl(s6),0,0,0,b(c))))*
		      XNf0(w,s6,0,0,c) + (4*log(z) - log(FA1(v(w),tl(s6),0,0,u(z),b(c))) - log(FA2(v(w),tl(s6),0,0,u(z),b(c))) - log(FB1(v(w),tl(s6),0,0,u(z),b(c))) - log(FB2(v(w),tl(s6),0,0,u(z),b(c))))*
		      XNf0(w,s6,0,z,c) + XNf1(w,s6,0,0,c) - XNf1(w,s6,0,z,c));
}

double NfI4E0u(double w,double s6,double z,double c) {
	return pow(mexp,-1)*pow(n,-1)*pow(z,-1)*(2*(-2*log(z)*(log(FA1(v(w),tl(s6),0,0,0,b(c))) + log(FA2(v(w),tl(s6),0,0,0,b(c))) + log(FB1(v(w),tl(s6),0,0,0,b(c))) + log(FB2(v(w),tl(s6),0,0,0,b(c)))) +
	        4*pow(log(z),2) + pow(log(FA1(v(w),tl(s6),0,0,0,b(c))),2) + pow(log(FA2(v(w),tl(s6),0,0,0,b(c))),2) + pow(log(FB1(v(w),tl(s6),0,0,0,b(c))),2) + pow(log(FB2(v(w),tl(s6),0,0,0,b(c))),2)
	        )*XNf0(w,s6,0,0,c) - 2*(-2*log(z)*(log(FA1(v(w),tl(s6),0,0,u(z),b(c))) + log(FA2(v(w),tl(s6),0,0,u(z),b(c))) + log(FB1(v(w),tl(s6),0,0,u(z),b(c))) +
	           log(FB2(v(w),tl(s6),0,0,u(z),b(c)))) + 4*pow(log(z),2) + pow(log(FA1(v(w),tl(s6),0,0,u(z),b(c))),2) + pow(log(FA2(v(w),tl(s6),0,0,u(z),b(c))),2) +
	        pow(log(FB1(v(w),tl(s6),0,0,u(z),b(c))),2) + pow(log(FB2(v(w),tl(s6),0,0,u(z),b(c))),2))*XNf0(w,s6,0,z,c) - 4*log(z)*XNf1(w,s6,0,0,c) +
	     log(FA1(v(w),tl(s6),0,0,0,b(c)))*XNf1(w,s6,0,0,c) + log(FA2(v(w),tl(s6),0,0,0,b(c)))*XNf1(w,s6,0,0,c) + log(FB1(v(w),tl(s6),0,0,0,b(c)))*XNf1(w,s6,0,0,c) +
	     log(FB2(v(w),tl(s6),0,0,0,b(c)))*XNf1(w,s6,0,0,c) + 4*log(z)*XNf1(w,s6,0,z,c) - log(FA1(v(w),tl(s6),0,0,u(z),b(c)))*XNf1(w,s6,0,z,c) -
	     log(FA2(v(w),tl(s6),0,0,u(z),b(c)))*XNf1(w,s6,0,z,c) - log(FB1(v(w),tl(s6),0,0,u(z),b(c)))*XNf1(w,s6,0,z,c) - log(FB2(v(w),tl(s6),0,0,u(z),b(c)))*XNf1(w,s6,0,z,c) + XNf2(w,s6,0,0,c) -
	     XNf2(w,s6,0,z,c));
}
