/*This file is part of SoftSERVE.
 *
 *   SoftSERVE is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   SoftSERVE is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with SoftSERVE.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * This file contains various auxiliary function definitions, as well as the structures collecting individuals terms 
 * integrated over a fixed number of variables into master formulae to be fed to the integrator.
 *
 * The definitions of the "physical parametrisation" variables (like b and y) in terms of "computing 
 * parametrisation" variables (like c and x) are listed here as well. This allows us to let the user specify the 
 * measurement function in the more compact and intuitive physical parametrisation. It also provides a convenient breaking 
 * point for the introduction of multi-precision variables in the case of the "boosted" branch. We know that any potentially  
 * problematic large cancellations occur inside the measurement function, and so it suffices to change any data types used  
 * for measurement function internal computations to multiprecision variables. Accordingly, we use the conversion functions  
 * to translate between "double" and the multi-precision "decimal" types:
 * The integrator provides a "double" type point in the "computing parametrisation"-parametrised integration space. The  
 * reparametrisation functions below convert these to a "decimal" type point in "physical parametrisation"-parametrised  
 * integration space (e.g. decimal b(double c) ). The measurement functions FXi accept these "decimal" type coordinates,  
 * perform internal computations using multi-precision operations, and return a "double" type result to be used in the full,  
 * "double"-valued, integrand function.
 *
 * Additionally, to spare the user the task of having to remember which definition of the angular variables is used and in  
 * which sector Xi of the measurement functions FXi, we define the cosini Ck and Cl of relevant angles betweek \vec{k_T} and  
 * \vec{l_T} and the one allowed distinguished transverse direction \vec{n_T} as functions of the relevant computing  
 * parametrisation variables.
 * 
 *
 * Finally, we define the integrand structures fed to the integrator. These are best described using an example:
 * For the 3-dimensional integration of the Cf^2 structure, there are 4 different expressions to be integrated over 3 variables,  
 * depending on which variable out  of (y,q,t5,b) or their rescaled versions is integrated over, in addition to the always 
 * present t/s and tl/s6.
 * Each of these expressions has an eps-expansion, so I get a sum of 4 terms for each appearing eps-order.
 * 
 * Cuba now requires the integrand in a peculiar form: It requires an integer-valued function with two arrays as arguments:  
 * x[nvar] and F[ncomp] (where nvar is the number of integration variables, and ncomp is the number of eps-orders, i.e.  
 * number of different sums of four terms above), and fills the array F with the values of the integrand structures, evaluated  
 * at the point with coordinates x. If this succeeds, the integer it returns is 1. A few auxiliary arguments are included for  
 * these functions, but these are irrelevant for our purposes.
 *
 * The notation is such that "D3Int" is the complete 'D'imension '3' 'Int'egrand, and contains all 
 * eps orders, ordered from least divergent (i.e. eps^0) in the first array entry F[0] to the most divergent (for Cf^2 eps^-4)
 * in the last array entry F[4]. 
 */

#include <Auxiliaries.h>
#include <cmath>
#include <string>
#include <2Int.h>
#include <3Int.h>
#include <4Int.h>
#include <5Int.h>
#include <XFactor.h>
#include <Input_Common.h>

using namespace std;

double mexp=M(m);

decimal tl(double s6) {
	return 4*pow(s6,4) - 6*pow(s6,8) + 4*pow(s6,12) - pow(s6,16);
}

decimal q(double r) {
	return pow(r,m);
}

decimal y(double x) {
	return pow(x,m);
}

decimal t5(double s5) {
	return pow(s5,2);
}


decimal t(double s) {
	return 4*pow(s,4) - 6*pow(s,8) + 4*pow(s,12) - pow(s,16);
}

decimal b(double c) {
	return pow(c,2);
}

decimal Tk1(decimal t, decimal t5, decimal tl) {
	decimal tk1;
	if (t>=1.) {
		tk1=1-tl;
	} else if (t<=0.) {
		tk1=tl;
	} else if (tl>=1.) {
		tk1=1-t;
	} else if (tl<=0.) {
		tk1=t;
	} else {
		tk1=t + tl - 2*t*tl - 2*(t5 - 1)*sqrt((1 - t)*t*(1 - tl)*tl);
	}
	return tk1;
}

decimal Tk2(decimal t, decimal t5, decimal tl) {
	decimal tk2;
	if (t>=1.) {
		tk2=1-tl;
	} else if (t<=0.) {
		tk2=tl;
	} else if (tl>=1.) {
		tk2=1-t;
	} else if (tl<=0.) {
		tk2=t;
	} else {
		tk2=t + tl - 2*t*tl - 2*(1 - t5)*sqrt((1 - t)*t*(1 - tl)*tl);
	}
	return tk2;
}

decimal Ck1(decimal t, decimal t5, decimal tl) {
	decimal ck1;
	if (t>=1.) {
		ck1=2.*tl-1.;
	} else if (t<=0.) {
		ck1=1.-2.*tl;
	} else if (tl>=1.) {
		ck1=2.*t-1.;
	} else if (tl<=0.) {
		ck1=1.-2.*t;
	} else {
		ck1=1.-2.*(t + tl - 2*t*tl + 2*(1 - t5)*sqrt((1 - t)*t*(1 - tl)*tl));
	}
	return ck1;
}

decimal Ck2(decimal t, decimal t5, decimal tl) {
	decimal ck2;
	if (t>=1.) {
		ck2=2.*tl-1.;
	} else if (t<=0.) {
		ck2=1.-2.*tl;
	} else if (tl>=1.) {
		ck2=2.*t-1.;
	} else if (tl<=0.) {
		ck2=1.-2.*t;
	} else {
		ck2=1.-2.*(t + tl - 2*t*tl - 2*(1 - t5)*sqrt((1 - t)*t*(1 - tl)*tl));
	}
	return ck2;
}

decimal Cl(decimal tl) {
	return 1.-2.*tl;
}

decimal Ck(decimal tk) {
	return 1.-2.*tk;
}

//get FXi from FX to hide structures from users, also swap q and r here (I used q for yk/yl, Guido used r, so internally I still use q, but G is defined using r)
double FA1a(decimal q,decimal y,decimal tl,decimal t,decimal t5,decimal b) {
	return ReturnDouble(1.+b)*GAa(q,y,tl,t,Tk1(t,t5,tl),b);
}

double FA1b(decimal q,decimal y,decimal tl,decimal t,decimal t5,decimal b) {
	return ReturnDouble(1.+b)*GAb(q,y,tl,t,Tk1(t,t5,tl),b);
}

double FA2a(decimal q,decimal y,decimal tl,decimal t,decimal t5,decimal b) {
	return ReturnDouble(1.+b)*GAa(q,y,tl,t,Tk2(t,t5,tl),b);
}

double FA2b(decimal q,decimal y,decimal tl,decimal t,decimal t5,decimal b) {
	return ReturnDouble(1.+b)*GAb(q,y,tl,t,Tk2(t,t5,tl),b);
}

//get exponent M from variable m to hide structures from users
double M(double m) {
	double mm=2/m;
	if (isnan(mm) || isinf(mm) || mm==0. || mm<=0.) {
		return 2.;
	} else {
		return 2/m;
	}	
}

int D2Int(const int* ndim, const double x[], const int* ncomp, double F[],void *userdata) {
	F[0]=I2E0(x[0],x[1]); //eps^0
	F[1]=I2EM1(x[0],x[1]); //eps^-1
	F[2]=I2EM2(x[0],x[1]); //eps^-2
	F[3]=I2EM3(x[0],x[1]); //eps^-3
	F[4]=I2EM4(x[0],x[1]); //eps^-4
	return 1;
}

int D3Int(const int* ndim, const double x[],const int* ncomp, double F[],void* userdata) {
	F[0]=I3E0r(x[0],x[1],x[2])+I3E0x(x[0],x[1],x[2])+I3E0c(x[0],x[1],x[2])+I3E0s(x[0],x[1],x[2]); //eps^0
	F[1]=I3EM1r(x[0],x[1],x[2])+I3EM1x(x[0],x[1],x[2])+I3EM1c(x[0],x[1],x[2])+I3EM1s(x[0],x[1],x[2]); //eps^-1
	F[2]=I3EM2r(x[0],x[1],x[2])+I3EM2x(x[0],x[1],x[2])+I3EM2c(x[0],x[1],x[2])+I3EM2s(x[0],x[1],x[2]); //eps^-2
	F[3]=I3EM3r(x[0],x[1],x[2])+I3EM3x(x[0],x[1],x[2])+I3EM3c(x[0],x[1],x[2])+I3EM3s(x[0],x[1],x[2]); //eps^-3
	F[4]=0; //eps^-4
	return 1;
}

int D4Int(const int* ndim, const double x[],const int* ncomp, double F[],void* userdata) {
	F[0]=I4E0rx(x[0],x[1],x[2],x[3])+I4E0rc(x[0],x[1],x[2],x[3])+I4E0rs(x[0],x[1],x[2],x[3])+I4E0xc(x[0],x[1],x[2],x[3])+I4E0xs(x[0],x[1],x[2],x[3])+I4E0cs(x[0],x[1],x[2],x[3]); //eps^0
	F[1]=I4EM1rx(x[0],x[1],x[2],x[3])+I4EM1rc(x[0],x[1],x[2],x[3])+I4EM1rs(x[0],x[1],x[2],x[3])+I4EM1xc(x[0],x[1],x[2],x[3])+I4EM1xs(x[0],x[1],x[2],x[3])+I4EM1cs(x[0],x[1],x[2],x[3]); //eps^-1
	F[2]=I4EM2rx(x[0],x[1],x[2],x[3])+I4EM2rc(x[0],x[1],x[2],x[3])+I4EM2rs(x[0],x[1],x[2],x[3])+I4EM2xc(x[0],x[1],x[2],x[3])+I4EM2xs(x[0],x[1],x[2],x[3])+I4EM2cs(x[0],x[1],x[2],x[3]); //eps^-2
	F[3]=0; //eps^-3
	F[4]=0; //eps^-
	return 1;
}

int D5Int(const int* ndim, const double x[],const int* ncomp, double F[],void* userdata) {
	F[0]=I5E0r(x[0],x[1],x[2],x[3],x[4])+I5E0x(x[0],x[1],x[2],x[3],x[4])+I5E0c(x[0],x[1],x[2],x[3],x[4])+I5E0s(x[0],x[1],x[2],x[3],x[4]); //eps^0
	F[1]=I5EM1r(x[0],x[1],x[2],x[3],x[4])+I5EM1x(x[0],x[1],x[2],x[3],x[4])+I5EM1c(x[0],x[1],x[2],x[3],x[4])+I5EM1s(x[0],x[1],x[2],x[3],x[4]); //eps^-1
	F[2]=0; //eps^-2
	F[3]=0; //eps^-3
	F[4]=0; //eps^-4
	return 1;
}
